local ADDON_NAME, Core = ...

--<
local date, time = date, time
local sub, gsub = string.sub, string.gsub
local upper, format, find, match = string.upper, string.format, string.find, string.match
local RGBToColorCode = RGBToColorCode
local UI = Core.UI
local utils = Core.utils
local Sessions = Core.Sessions
-->

local myCharacterName, myBattleTag, myRPAlias, myGUID
local cache = {}


local formatPatterns = {
	generic = "%s%s: %s",
	emoteApostrophe = "%s%s%s",
	emoteSpecial = [[%s%s|TInterface\FriendsFrame\BroadcastIcon:16:16|t %s]],
	EMOTE = "%s* %s %s",

	verbose = {
		WHISPER = "%s%s whispers: %s",
		WHISPER_INFORM = "%sTo %s: %s",
		BN_WHISPER = "%s%s whispers: %s",
		BN_WHISPER_INFORM = "%sTo %s: %s",
		SAY = "%s%s says: %s",
	}
}

setmetatable(formatPatterns, { __index = function(self) return self.generic end })
Core.formatPatterns = formatPatterns


local function chatMsgRestrictedHandler (...)
	local err, target = ...
	local S = Sessions:Get(target)

	if err == "ChatIgnoredHandler" and S then
		S.page:AddMessage(_G.CHAT_RESTRICTED, 1, 0, 0)
	end
end



local function sendEmote (cmd, msg, session)
	if msg == nil and not UnitName("target") and session then
		msg = session:GetName()
	end
	
	cmd = gsub(cmd, "/", "")
	
	DoEmote(cmd, msg)
end


local function bnetFriendUpdateHandler (...)
	local friendIndex = ...
	if friendIndex == nil then 
		return 
	end
	
	local payload = C_BattleNet.GetFriendAccountInfo(friendIndex)
	
	local S = Sessions:Get(payload.bnetAccountID)
	if not S then return end
	
	Core.BNetDisplayUpdate(S.tab, payload.gameAccountInfo)
end


local function replyHook ()
	if Sessions.lastWhisperTarget == nil then
		return
	end	

	if not UI:IsShown() then
		UI:Show()
	end

	Sessions:Switch(Sessions.lastWhisperTarget)
	UI:FocusEditBoxFromKeybind()
end



local function updateHeaderHook (editBox)
	if editBox:GetAttribute("chatType"):match("WHISPER") == nil then
		Core.G_ChatType = editBox:GetAttribute("chatType")
	end	
end


local function insertItemLink (item)
	if UI.editBox:HasFocus() and IsModifiedClick("CHATLINK") then
		local loc = PaperDollFrame:IsMouseOver() and ItemLocation:CreateFromEquipmentSlot(item:GetID()) or ItemLocation:CreateFromBagAndSlot(item:GetParent():GetID(), item:GetID())
		UI.editBox:Insert(C_Item.GetItemLink(loc))
		
		if StackSplitFrame:IsShown() then
			StackSplitFrame:Hide()
		end
	end
end


local function extractTellTargetHook (editBox, msg)
	local target = msg:match("%s*(.*)")
	
	if ( not target or not target:find("%s") ) then
		return false;
	end
	
	if target:sub(1, 1) == "|" then
		return false
	end
	
	while target:find("%s") do
		target = target:match("(.+)%s+[^%s]*")
		if #GetAutoCompleteResults(target, 1, 0, true, AUTOCOMPLETE_LIST.ALL.include,
			AUTOCOMPLETE_LIST.ALL.exclude) > 0 then 
			break
		end
		
	end

	Sessions:MatchTargetName(target)
	
	UI:Show()
	editBox:Hide()
	UI.editBox:SetFocus()
	
end


local function sendBNetTellHook (name)
	Core.G_EditBox:Hide()
	UI:Show()
	Sessions:MatchTargetName(name)
end


local function onPortraitsUpdated ()
	if #Core.orderedTabs == 0 or InCombatLockdown() or not IsInGroup(LE_PARTY_CATEGORY_HOME) then
		return
	end
	
	local numPartyMembers = GetNumGroupMembers(LE_PARTY_CATEGORY_HOME)-1
	
	if numPartyMembers > 0 then		
		for i=1, numPartyMembers do
			local unit = "party".. tostring(i)
			local S = Sessions:Get(UnitGUID(unit))
			
			if S and UnitIsVisible(unit) then
				S:UpdatePortrait(unit)
			end
		end
	end
end


local function GetPattern (ctype, msg)
	if ctype == "EMOTE" and (msg:sub(1, 1) == "|" or msg:match("^%u")) then
		return formatPatterns.emoteSpecial
	elseif ctype == "EMOTE" and msg:sub(1, 1) == "'" then
		return formatPatterns.emoteApostrophe
	else
		return formatPatterns[ctype]
	end
end


local function ParseText (ctype, msg)
	local pattern, hex
	
	if ctype ~= "SAY" then
		pattern = '%b""'; hex = RGBToColorCode(utils.GetRGB("SAY"))
	elseif ctype ~= "EMOTE" then
		pattern = '%b**'; hex = RGBToColorCode(utils.GetRGB("EMOTE"))
	end
	
	msg = gsub(msg, pattern, function(catch)
		local _, pos = msg:find(catch)
		
		if pos and msg:sub(pos+1, pos+2) == "|r" then
			return catch
		else
			return format("%s%s|r", hex, catch)
		end		
	end)
	
	if ctype ~= "EMOTE" then
		msg = gsub(msg, "<?https?://[%w%p]+", function(url)
			if url:match("<") then
				return url
			else
				return format("|H%s|h|cFFC845FA<%s>|r|h", url, url)
			end
		end)
	end
	
	return msg
end


local recentLongMessages = {}

local function onChatMessageReceived (e, ...)
	local ctype = e:gsub("CHAT_MSG_", "")
	
	if ctype == "ADDON" then
		if ... ~= "SessionsMSGR" then
			return
		end
		
		local _, cmd, _, sender = ...
		local S = Sessions:Get(utils.getFullName(sender))
		
		if not S then
			return 
		elseif cmd == "TI01" then
			S:SetTyping()
		end
		
		if not S.isUsingSM then
			S.isUsingSM = true
			UI.editBox:AddOrRemoveOnTextHandler()
		end
		
		return
		
	elseif ctype == "RESTRICTED" then
		return chatMsgRestrictedHandler(...)
	end
	
	local msg, author, language, _, _, _, _, chanIndex, _, _, lineID, GUID, bnetID = ...
	local inbound = (not ctype:match("INFORM$") and true) or false
	local now = date("%I:%M ")
	local S, isBnet

	-- Get or create the appropriate session 
	
	if ctype:match("^WHISPER") then
		S = Sessions:Get(author)
		if not S then S = Sessions.CreateFromGUID(GUID) end
	
	elseif ctype == "AFK" or ctype == "DND" then
		S = Sessions:Get(GUID)
		if not S then return end
		local gstr = _G["CHAT_".. ctype.. "_GET"]
		S.page:AddMessage(format(gstr, author).. msg)
		return
	
	elseif ctype:match("^BN_") then
		S = Sessions:Get(bnetID); isBnet = true
		if not S then S = Sessions.CreateFromID(bnetID) end
	else 
		S = Sessions:Get(ctype)
		if not S then S = Sessions.CreateCustom(unpack(Sessions.preset[ctype])) end
	end

	Sessions:UpdateIfNeeded(author, GUID)

	-- First, check if we're appending to the author's previous fontString and exiting

	if recentLongMessages[author] then
		if recentLongMessages[author] >= GetTime() then
			for i=1, 3 do
				local info = S.page.historyBuffer:GetEntryAtIndex(i)

				if info.extraData[1] == author then
					if info.extraData.n == 1 then
						info.message = ParseText(ctype, info.message.. msg)
						S.page:RefreshLayout()
					else
						info.extraData[2] = ParseText(ctype, info.extraData[2].. msg)
					end
					
					if S.isDM then
						S:AddHistory(inbound, isBnet, now, msg)
					end
					
					break
				end
			end

			return			
		else
			recentLongMessages[author] = nil
		end
	end
	
	if #msg > 240 then
		recentLongMessages[author] = GetTime() + 2
	end
	
	-- Create the appropriate playerLink for this context
	
	local typePreferences = Sessions.preferences[ctype]
	
	local trueinbound = inbound or Core.db.UseVerbose
	local playerLink, displayName, alias
	
	if isBnet then
		playerLink = (inbound or Core.db.UseVerbose) and GetBNPlayerLink(author, author, bnetID) or myBattleTag
		
	elseif trueinbound then
		if typePreferences.useRPAlias then
			alias = Sessions:GetRoleplayAlias(author)
		end
		displayName = Ambiguate(author, "none")
	else
		if typePreferences.useRPAlias then
			alias = myRPAlias
		end
		displayName = myCharacterName
	end
	
	if not playerLink then
		local finalName
		
		if (typePreferences.nameColoured and not alias) or (typePreferences.nameColoured and alias and not alias:match("|r")) then
			local _, englishClass = GetPlayerInfoByGUID(trueinbound and GUID or myGUID)
			local classColour = RAID_CLASS_COLORS[englishClass]
			
			if classColour then
				finalName = classColour:WrapTextInColorCode(alias or displayName)
			end
		else
			finalName = alias or displayName
		end
		
		playerLink = GetPlayerLink(trueinbound and author or myCharacterName, finalName)
	end

	if Core.db.BracketedNames and not ctype:match("EMOTE") then playerLink = format("[%s]", playerLink) end
	
	-- Get the timestamp
	
	local ts
	
	if not typePreferences.tsEnabled then
		ts = ""
	else
		ts = (typePreferences.tsColoured and format("|cFFB3B3B3%s|r", now)) or now
	end
	
	-- Get the appropriate RGB for this context
	
	local r, g, b

	if not inbound and ctype ~= "PARTY_LEADER" then
		r, g, b = .85, .85, .85
	else
		r, g, b = utils.GetRGB(ctype)
	end
	
	-- Put the pieces together!
	
	local body
	
	if ctype == "TEXT_EMOTE" then
		body = format("%s%s", ts, msg:gsub(author, playerLink))
	else
		body = ParseText(ctype, msg)
		body = format(GetPattern(ctype, msg), ts, playerLink, body)
	end

	if S.selectiveModeEnabled and not Sessions:Get(author) and GUID ~= myGUID then
		local notice
		
		if match(msg, myCharacterName) then
			notice = [[|TInterface\OPTIONSFRAME\UI-OptionsFrame-NewFeatureIcon:14:14|t ]]
		end
		
		S.page:AddMessage(
			format("%s%s[by %s: Right-Click To Show]|r", notice or "", RGBToColorCode(.7, .7, .7), author),
			r, g, b, author, body
		)
	else
		if S.isUsingSM then
			S.page:ClearTypingStatus()
		end
		
		S.page:AddMessage(body, r, g, b, author)
	end

	-- Lastly, handle notifications and interface things
	
	S.page:Update()
	
	if S.isDM then
		if Core.db.HistoryExpiry > 0 then
			S:AddHistory(inbound, isBnet, now, msg)
		end
		
		if S.isMuted then return end
		
		if UI:IsShown() then
			if not S:IsSelected() then
				S:UpdateNotice(ctype)
				S:StartGlowing()
			end
			
		elseif InCombatLockdown() or Core.db.KeepMinimized then
			if inbound then
				S:UpdateNotice(ctype)
			end
		else
			Sessions:Switch(S)
			UI:Show()
		end
		
		if inbound or Sessions.lastWhisperTarget == nil then
			Sessions.lastWhisperTarget = S
			PlaySound(3081, "Master")
			FlashClientIcon()
		end
		
		if S.tab.tabIndex ~= 1 then
			table.insert(Core.orderedTabs, 1, table.remove(Core.orderedTabs, S.tab.tabIndex))
			Core.Tab_ReanchorAll()
		end
		
		if not isBnet and S.isUsingSM == nil then
			S:Handshake()
		end
		
	elseif (not S:IsSelected() or not UI:IsShown()) and not S.isMuted then
		S:UpdateNotice(ctype)
	end
end



UI:SetScript("OnEvent", function(self, e, ...)
	if e:match("CHAT_MSG_") then
		onChatMessageReceived(e, ...)
	
	elseif e == "BN_FRIEND_INFO_CHANGED" then
		bnetFriendUpdateHandler(...)

	elseif e == "GROUP_JOINED" then
		local S = Sessions:Get("PARTY") or Sessions.CreateCustom("Party", "PARTY")
		S.page:AddMessage(format(_G.ERR_JOINED_GROUP_S, myCharacterName), 1, 1, 0)
		
	elseif e == "GROUP_LEFT" then
		local S = Sessions:Get("PARTY")
		if S then S.page:AddMessage(_G.ERR_LEFT_GROUP_YOU, 1, 0, 0) end

	elseif e == "MODIFIER_STATE_CHANGED" and UI.main:IsShown() then
		local key, state = ...
		if key ~= "LCTRL" then return end
		
		if Sessions.selected ~= nil then
			Sessions.selected.page:SetTextCopyable(state > 0 and 1 or nil)
		end
		--[[
		if state == 1 then
			self.resizeBtn:Show()
			
		else
			self.main:StopMovingOrSizing()
			self.resizeBtn:Hide()
		end
		--]]
	elseif e == "PLAYER_TARGET_CHANGED" then
		if not UnitIsPlayer("target") or not UnitIsVisible("target") then
			return
		end
		
		local S = Sessions:Get(UnitGUID("target"))

		if S then
			S:UpdatePortrait("target")
		end
	
	elseif (e == "GROUP_ROSTER_UPDATE" and not IsInRaid()) or (e == "PORTRAITS_UPDATED") then
		onPortraitsUpdated()
		
	elseif e == "WHO_LIST_UPDATE" then
		Sessions:OnWhoReceived()
		
	elseif e:match("ZONE_CHANGED") and Sessions:Get("NEARBY") then
		Sessions:Get("NEARBY"):UpdateTitle()
	end	
end)



--[[
UI.leftFrame.inputBox:HookScript("OnEnterPressed", function(self)
	if self:GetText() ~= "" then
		onTellTargetIntercept(self:GetText())
	end
	
	self:SetText("") ; self:ClearFocus()
end)


UI.leftFrame.inputBox:HookScript("OnTextChanged", function(self, byUser)
	if not byUser then
		return
	end
	
	local text = utils.capitalize(self:GetText())
	
	Core.suggestionFrame:ClearList()
	
	if text == "" then 
		return Core.suggestionFrame:Hide() 
	end
	
	Core.suggestionFrame:Show()
	
	local results = GetAutoCompleteResults(text, 4, self:GetCursorPosition(), true, AUTOCOMPLETE_FLAG_ONLINE, AUTOCOMPLETE_FLAG_NONE)

	for i, v in ipairs(results) do
		if Core.suggestionFrame:HasSpace() then
			Core.suggestionFrame:Add(Ambiguate(v.name, "none"), function() onTellTargetIntercept(v.name); self:SetText("") end)
		end
	end
	
end)
]]


hooksecurefunc("ChatEdit_ExtractTellTarget", extractTellTargetHook)
hooksecurefunc("ChatEdit_UpdateHeader", updateHeaderHook)
hooksecurefunc("ChatFrame_ReplyTell", replyHook)
hooksecurefunc("ChatFrame_ReplyTell2", replyHook)
--hooksecurefunc("ChatFrame_SendTell")
hooksecurefunc("ChatFrame_SendBNetTell", sendBNetTellHook)
--hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", insertItemLink)
--hooksecurefunc("PaperDollItemSlotButton_OnModifiedClick", insertItemLink)


local function debug ()
	if Core.debugging then
		for i=1, #Core.debugging do
			local err = Core.debugging[i]
			utils.SMPrint(err)
		end
	end
end



SlashCmdList["SESSIONS_TALKTO"] = function(msg)
	onTellTargetIntercept(msg)
	UI:Show()
end

SLASH_SESSIONS_TALKTO1 = "/talkto"
SLASH_SESSIONS_TALKTO2 = "/tt"


SlashCmdList["SESSIONS_DEBUG"] = debug

SLASH_SESSIONS_DEBUG1 = "/smdebug"

function Core.UpdatePlayerInfo ()
	myCharacterName = UnitName("Player")
	myGUID = UnitGUID("Player")
	--myRPAlias = Sessions:GetRoleplayAlias(myCharacterName)
	myBattleTag = BNConnected() and BNet_GetTruncatedBattleTag(select(2, BNGetInfo())) or "Me"
	Core.myCharacterName = myCharacterName
	Core.myRealmName = gsub(GetRealmName(), " ", "")
	
	Core.myInfo.character = myCharacterName
	Core.myInfo.battleTag = myBattleTag
	Core.myInfo.fullName = myCharacterName.. "-".. Core.myRealmName
	Core.myInfo.GUID = myGUID
	Core.myInfo.guildName = GetGuildInfo("player")
end

